import logging
import sys


def getLogger(name: str, log_level=logging.INFO) -> logging.Logger:
    """Returns a logger with a default format."""

    handler = logging.StreamHandler(stream=sys.stdout)
    handler.setLevel(log_level)
    handler.setFormatter(
        logging.Formatter(
            fmt="%(asctime)s.%(msecs)03d | %(levelname)s | %(name)s | %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )
    )

    logger = logging.getLogger(name)
    logger.addHandler(handler)
    logger.setLevel(log_level)

    return logger



import threading
import time

import pytest

from caniusethat.shareable import (
    Server,
    _force_remote_server_stop,
    acquire_lock,
    release_lock,
    you_can_use_this,
)
from caniusethat.thing import Thing

SERVER_ADDRESS = "tcp://127.0.0.1:6555"


class ClassWithoutLocks:
    @you_can_use_this
    def add(self, a: int, b: int) -> int:
        """Add two numbers."""
        return a + b


class ClassWithLocks:
    @you_can_use_this
    @acquire_lock
    def initialize(self) -> None:
        """Initialize the class."""
        pass

    @you_can_use_this
    def add(self, a: int, b: int) -> int:
        """Add two numbers."""
        return a + b

    @you_can_use_this
    @release_lock
    def finalize(self) -> None:
        """Finalize the class."""
        pass


@pytest.fixture(autouse=True)
def wait_for_context_cleanup():
    yield
    time.sleep(0.1)


def test_local_server_shutdown():
    my_obj = ClassWithoutLocks()

    my_server = Server(SERVER_ADDRESS)
    my_server.start()
    my_server.add_object("my_obj", my_obj)
    time.sleep(0.5)
    my_server.stop()


def test_remote_server_shutdown():
    my_obj = ClassWithoutLocks()

    my_server = Server(SERVER_ADDRESS)
    my_server.start()
    my_server.add_object("my_obj", my_obj)
    _force_remote_server_stop(SERVER_ADDRESS)
    my_server.join()


def test_shared_thing():

    my_obj = ClassWithoutLocks()

    my_server = Server(SERVER_ADDRESS)
    my_server.start()
    my_server.add_object("my_obj", my_obj)
    time.sleep(0.5)

    my_thing = Thing("my_obj", SERVER_ADDRESS)
    assert hasattr(my_thing, "add")
    assert not hasattr(my_thing, "multiply")
    assert my_thing.add(1, 2) == 3
    assert my_thing.add(10, 20) == 30
    assert my_thing.add.__doc__ == "(a: int, b: int) -> int\nAdd two numbers."
    with pytest.raises(RuntimeError, match="RemoteProcedureError.METHOD_EXCEPTION"):
        my_thing.add(1, 2, 3)

    _force_remote_server_stop(SERVER_ADDRESS)

    my_server.join()


def test_locked_shared_thing():
    my_obj = ClassWithLocks()

    my_server = Server(SERVER_ADDRESS)
    my_server.start()
    my_server.add_object("my_obj", my_obj)
    time.sleep(0.5)

    my_thing = Thing("my_obj", SERVER_ADDRESS)
    assert hasattr(my_thing, "initialize")
    assert hasattr(my_thing, "add")
    assert hasattr(my_thing, "finalize")
    my_thing.initialize()
    assert my_thing.add(1, 2) == 3
    my_thing.finalize()

    _force_remote_server_stop(SERVER_ADDRESS)

    my_server.join()



from setuptools import setup

with open("requirements.txt") as installation_requirements_file:
    requirements = installation_requirements_file.read().splitlines()

with open("requirements-dev.txt") as test_requirements_file:
    test_requirements = test_requirements_file.read().splitlines()

with open("README.md") as readme_file:
    readme = readme_file.read()

project_urls = {
    "Home": "https://github.com/matpompili/caniusethat",
    "Documentation": "https://caniusethat.readthedocs.io/",
    "Changelog": "https://github.com/matpompili/caniusethat/blob/main/CHANGELOG.md",
}

setup(
    name="caniusethat",
    version="0.2.0-beta.0",
    packages=["caniusethat"],
    author="Matteo Pompili",
    python_requires=">=3.8",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    description="TODO.",
    long_description=readme,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    test_suite="tests",
    tests_require=test_requirements,
    package_data={
        "": ["LICENSE"],
        "caniusethat": ["py.typed"],
    },
    project_urls=project_urls,
)



from enum import Enum, auto
from typing import Any, Dict, List, NamedTuple, Optional, Tuple


class SharedMethodDescriptor(NamedTuple):
    name: str
    signature: str
    docstring: str


class SharedObjectDescriptor(NamedTuple):
    name: str
    object: Any
    shared_methods: List[SharedMethodDescriptor]
    locking_methods: List[str]
    unlocking_methods: List[str]


class RemoteProcedureCall(NamedTuple):
    name: str
    method: str
    args: Tuple[Any, ...] = ()
    kwargs: Dict[str, Any] = {}


class RemoteProcedureError(Enum):
    NO_ERROR = auto()
    NO_SUCH_THING = auto()
    NO_SUCH_METHOD = auto()
    METHOD_EXCEPTION = auto()
    INVALID_RPC = auto()
    THING_IS_LOCKED = auto()


class RemoteProcedureResponse(NamedTuple):
    result: Any
    error: RemoteProcedureError
